@@ -50,13 +50,15 @@ import os
 import subprocess
 import traceback
 from io import BytesIO
+import pkg_resources
+
 import streamlit as st
 import pandas as pd
 import plotly.express as px
 
 # from terminal, popen has different paths, so adding the package path manually
 sys.path.append(os.path.dirname(__file__) + '/../')
-import pandas_eda
+import pandas_eda.explore
 
 pd.options.plotting.backend = "plotly"
 
@@ -100,6 +102,7 @@ class Main:
         self.small_table_to_show = None
         self.mem_size = None
         self.plot_hist = None
+        self.preserve_column_order_at_sidebar = None
 
         st.set_page_config(layout="wide")
 
@@ -118,7 +121,8 @@ class Main:
         self.show_data()
         self.show_statistics()
         self.show_sizes()
-        self.show_frequent_values()
+        self.show_frequent_values_at_main()
+        self.show_frequent_values_at_sidebar()
         self.help()
 
         print('done reading table')
@@ -132,6 +136,7 @@ class Main:
         with self.tab_config:
             columns = st.columns(3)
             self.plot_hist = columns[0].checkbox('plot histogram at numeric columns')
+            self.preserve_column_order_at_sidebar = columns[0].checkbox('preserve column order at sidebar')
             self.number_of_most_frequent_values = columns[0] \
                 .slider('number of most frequent values to display', 2, 10, 6)
             columns_to_drop = columns[0].multiselect('select columns to ignore', self.df.columns.tolist())
@@ -143,10 +148,14 @@ class Main:
             if len(query):
                 try:
                     self.df = self.df.query(query)
+                    if self.df.empty:
+                        st.sidebar.error('filtering data, at config tab, left empty table!!')
+                        st.stop()
                     st.sidebar.warning('showing filtered data by the query at the config tab!')
-                except (pd.errors.UndefinedVariableError, SyntaxError):
-                    st.error('cannot use this query due to:')
+                except (pd.errors.UndefinedVariableError, ValueError, SyntaxError):
+                    st.error('query has errors:')
                     st.code(traceback.format_exc())
+                    st.stop()
 
     def help(self):
         with self.tab_help:
@@ -194,32 +203,36 @@ class Main:
             st.code(f'{self.mem_size.sum():.3f} MB\n{self.df.shape[0]:,} rows\n{self.df.shape[1]:,} columns')
 
             st.subheader('size of each column:')
-            columns = st.columns([1, 3])
+            columns = st.columns([1, 2])
             columns[0].table(self.mem_size.to_frame().style.bar(color='#ead9ff').format('{:.3f}'))
             fig = px.pie(self.mem_size.to_frame().reset_index(), names='index', values='MB', hole=0.4)
             fig.update_traces(texttemplate="%{label}:<br>%{value:,.3f}MB<br>%{percent:.0%}", )
+            if columns[1].checkbox('remove labels from pie chart', value=self.mem_size.shape[0]>10):
+                fig.update_traces(texttemplate='', textinfo="none", )
             columns[1].plotly_chart(fig)
 
-    def show_frequent_values(self):
+    def show_frequent_values_at_main(self):
         with self.tab_frequent_values:
             st.subheader('frequent values:')
             frequent = self.eda.get_frequent_values()
             frequent.value = frequent.value.astype(str)  # streamlit issue
             download(frequent, 'frequent_values')
 
-        # now the sidebar
+    def show_frequent_values_at_sidebar(self):
         st.sidebar.header('frequent values per column:')
         freq = self.eda.get_frequent_values()
-        for col in freq.col.unique():
+        columns = freq.col.unique()
+        if self.preserve_column_order_at_sidebar:
+            columns = self.df.columns[self.df.columns.isin(columns)].values
+        for col in columns:
             single_col_statistics = self.columns_statistics.astype(dict(col=str))
             single_col_statistics = single_col_statistics.query(f'col=="{col}"').iloc[0]
-
-            st.sidebar.info(f'{col} [{single_col_statistics.uniques:,} uniques, {single_col_statistics.nans:,} nans]')
             show = freq.query(f'col=="{col}"').set_index('value')
-            for index, row in show.iterrows():
-                st.sidebar.write(f'{index}: [#{row.counts:,}, {row.counts / self.df.shape[0]:.2%}]')
-                st.sidebar.progress(row.percentages)
 
+            # column name, uniques and nans
+            st.sidebar.info(f'{col} [{single_col_statistics.uniques:,} uniques, {single_col_statistics.nans:,} nans]')
+
+            # histogram
             if self.plot_hist:
                 if pd.api.types.is_numeric_dtype(self.df[col]) and single_col_statistics.uniques > show.shape[0]:
                     bins = st.sidebar.slider('number of bins',
@@ -230,11 +243,20 @@ class Main:
                     bins = bins if bins > 1 else None
                     st.sidebar.plotly_chart(self.df.plot.hist(x=col, height=300, nbins=bins),
                                             use_container_width=True)
+            # frequent values
+            for index, row in show.iterrows():
+                st.sidebar.write(f'{index}: [#{row.counts:,}, {row.counts / self.df.shape[0]:.2%}]')
+                st.sidebar.progress(row.percentages)
+
+
 
 
 if __name__ == '__main__':
-    if st._is_running_with_streamlit:
-        # this code cannot really work as we dont have sys.argv[1] that contains the table to show...
+    if pkg_resources.parse_version(st.__version__).release > (1, 13, 0):
+        already_running = st.runtime.exists()
+    else:
+        already_running = st._is_running_with_streamlit
+    if already_running:
         Main()
     else:
         argv = ["streamlit", "run", sys.argv[0]]
